@@ -1348,7 +1348,9 @@ class GzipTestCase(SimpleHandlerTestCase):
             self.write('hello world')
 
     def get_app_kwargs(self):
-        return dict(gzip=True)
+        return dict(
+            gzip=True,
+            static_path=os.path.join(os.path.dirname(__file__), 'static'))
 
     def test_gzip(self):
         response = self.fetch('/')
@@ -1361,6 +1363,17 @@ class GzipTestCase(SimpleHandlerTestCase):
             'gzip')
         self.assertEqual(response.headers['Vary'], 'Accept-Encoding')
 
+    def test_gzip_static(self):
+        # The streaming responses in StaticFileHandler have subtle
+        # interactions with the gzip output so test this case separately.
+        response = self.fetch('/robots.txt')
+        self.assertEqual(
+            response.headers.get(
+                'Content-Encoding',
+                response.headers.get('X-Consumed-Content-Encoding')),
+            'gzip')
+        self.assertEqual(response.headers['Vary'], 'Accept-Encoding')
+
     def test_gzip_not_requested(self):
         response = self.fetch('/', use_gzip=False)
         self.assertNotIn('Content-Encoding', response.headers)
